function love.load()
	love.graphics.setBackgroundColor(139, 179, 213)
	
	level = {}
	level.world = love.physics.newWorld(0,0)
	level.world:setCallbacks(beginContact, endContact, preSolve, postSolve)
	
	-- some constants
	gravity = 9.81
	gravityAddition = 100000
	meter = 1000
	jumpForce = meter * 20000
	
	-- create 3 planets
	newPlanet(2000,1000,  1000,  50,127,50)
	newPlanet(-2000,-1000,  1000,  127,50,50)
	newPlanet(0,-6000,  2000,  50,50,127)
	
	-- create some floating balls
	newBall(2000,2000, 100, 0,0,0)
	newBall(2000,2000, 100, 0,0,0)
	newBall(2000,2000, 100, 0,0,0)
	
	newBall(-3000,2000, 100, 0,0,0)
	newBall(-3000,2000, 100, 0,0,0)
	newBall(-3000,2000, 100, 0,0,0)
	
	newBall(-3000,-5000, 100, 0,0,0)
	newBall(-3000,-5000, 100, 0,0,0)
	newBall(-3000,-5000, 100, 0,0,0)
	
	-- create the ball you controle
	avatar = newBall(2000,2000, 500, 0,0,127)
	
	
	drawPositions = false	
end

--- create a planet
function newPlanet(x,y, size, r,g,b)
	local index = #level + 1
	level[index] = {}
	level[index].body = love.physics.newBody(level.world, x or 0, y or 0, "static")
	level[index].shape = love.physics.newCircleShape(size or 1000)
	level[index].fixture = love.physics.newFixture(level[index].body, level[index].shape)
	level[index].fixture:setUserData(level[index])
	level[index].role = "gravitator"
	level[index].color = {r or 255,g or 255,b or 255}
	
	return level[index]
end

-- create a floating ball
function newBall(x,y, size, r,g,b)
	local index = #level + 1
	level[index] = {}
	level[index].body = love.physics.newBody(level.world, x or 0, y or 0, "dynamic")
	level[index].shape = love.physics.newCircleShape(size or 1000)
	level[index].fixture = love.physics.newFixture(level[index].body, level[index].shape)
	level[index].fixture:setUserData(level[index])
	level[index].role = "gravitable"
	level[index].color = {r or 255,g or 255,b or 255}
	level[index].follower = nil

	return level[index]
end

-- draw every object in list, follow avatar-ball
function love.draw()
	love.graphics.push()
	love.graphics.translate(love.graphics.getWidth()/2, love.graphics.getHeight()/2)
	love.graphics.scale(1/10,1/10)
	love.graphics.translate(-avatar.body:getX(), -avatar.body:getY())
	for i,currentObject in pairs(level) do
		if i ~= "world" then
			
			-- set color
			if currentObject.color then
				love.graphics.setColor(unpack(currentObject.color))
			else
				love.graphics.setColor(255,255,255)				
			end
			
			-- draw shape
			if currentObject.shape:type() == "CircleShape" then
				love.graphics.circle("fill", currentObject.body:getX(), currentObject.body:getY(), currentObject.shape:getRadius())
			elseif currentObject.shape:type() == "PolygonShape" or currentObject.shape:type() == "RectangleShape" then
				love.graphics.polygon("fill", currentObject.body:getWorldPoints(currentObject.shape:getPoints()))
			end
			
			-- draw position point
			if drawPositions then
				local col = {love.graphics.getColor()}
				love.graphics.setColor(255 - col[1], 255 - col[2], 255 - col[3])
				love.graphics.setPointSize(10)
				love.graphics.point(currentObject.body:getX(), currentObject.body:getY())
				love.graphics.setPointSize(1)
			end
		end
	end
	
	love.graphics.pop()
end

function love.update(dt)
	
	-- physics loop over all gravitables and gravitators
	-- every gravitator causes a gravitation, that influences a gravitable
	
	for i,currentGravitable in pairs(level) do -- gravitables
		if i ~= "world" and currentGravitable.role == "gravitable" then
			
			local toApply = {}; toApply.x = 0; toApply.y = 0
			
			-- if there is a follower specified in the currentGravitator only follow this one
			-- otherwise follow every gravitator
			gravitators = level
			if currentGravitable.follower then
				gravitators = {currentGravitable.follower}
			end
			
			for j,currentGravitator in pairs(gravitators) do -- gravitators
				if j ~= "world" and currentGravitator.role == "gravitator" then
					
					local distanceDirX = currentGravitator.body:getX() - currentGravitable.body:getX()
					local distanceDirY = currentGravitator.body:getY() - currentGravitable.body:getY()
					local distLength = math.sqrt(distanceDirX*distanceDirX + distanceDirY*distanceDirY)
					
					distanceDirX = distanceDirX / distLength
					distanceDirY = distanceDirY / distLength
					
					local gravitatorGravity = 1
					if currentGravitator.shape:type() == "CircleShape" then
						gravitatorGravity = currentGravitator.shape:getRadius()*4
					end
					
					toApply.x = toApply.x + ((distanceDirX * gravitatorGravity * gravityAddition) / (distLength)) * dt
					toApply.y = toApply.y + ((distanceDirY * gravitatorGravity * gravityAddition) / (distLength)) * dt
				end
			end
			-- gravitators end
			
			currentGravitable.body:applyForce(toApply.x * gravity * meter, toApply.y * gravity * meter)
			
		end
	end
	-- gravitables end
	level.world:update(dt)
end

--- controls
function love.keypressed(key, unicode)
	if key == "a" or key == "left" then
		avatar.body:applyLinearImpulse(-jumpForce, 0)
	end
	
	if key == "d" or key == "right" then
		avatar.body:applyLinearImpulse(jumpForce, 0)
	end
	
	if key == "s" or key == "down" then
		avatar.body:applyLinearImpulse(0, jumpForce)
	end
	
	if key == "w" or key == "up" then
		avatar.body:applyLinearImpulse(0, -jumpForce)
	end
	
	avatar.follower = nil
	
	if key == "q" or key == "escape" then
		love.event.push("quit")
	end
end

function setFollower(gravitator, gravitable)
	gravitable.follower = gravitator
end

--- Physics-Callbacks
function beginContact(a, b, coll)
    objectA = a:getUserData()
	objectB = b:getUserData()
	
	if objectA.role == "gravitator" and objectB.role == "gravitable" then
		setFollower(objectA, objectB)
	elseif objectA.role == "gravitable" and objectB.role == "gravitator" then
		setFollower(objectB, objectA)
	end
end

function endContact(a, b, coll)
    
end

function preSolve(a, b, coll)
    
end

function postSolve(a, b, coll)
    
end

--- conf stuff
function love.conf(t)
    t.title = "Planet-Gravitation-Test"		-- The title of the window the game is in (string)
    t.author = "Steve Wolligandt"			-- The author of the game (string)
    t.identity = "test"						-- The name of the save directory (string)
    t.version = "0.8.0"						-- The LVE version this game was made for (string)
    t.console = false						-- Attach a console (boolean, Windows only)
    t.release = false						-- Enable release mode (boolean)
    t.screen.width = 800					-- The window width (number)
    t.screen.height = 600					-- The window height (number)
    t.screen.fullscreen = false				-- Enable fullscreen (boolean)
    t.screen.vsync = false					-- Enable vertical sync (boolean)
    t.screen.fsaa = 0						-- The number of FSAA-buffers (number)
    t.modules.joystick = true				-- Enable the joystick module (boolean)
    t.modules.audio = true					-- Enable the audio module (boolean)
    t.modules.keyboard = true				-- Enable the keyboard module (boolean)
    t.modules.event = true					-- the event module (boolean)
    t.modules.image = true					-- Enable the image module (boolean)
    t.modules.graphics = true				-- Enable the graphics module (boolean)
    t.modules.timer = true					-- Enable the timer module (boolean)
    t.modules.mouse = true					-- Enable the mouse module (boolean)
    t.modules.sound = true					-- Enable the sound module (boolean)
    t.modules.physics = true				-- Enable the physics module (boolean)
end
